% Plot up the Transmon table
close all
clear classes

disp(' Transmon table.')
disp(' ')


%% Parameters
% Our seedwaist comes from the arm cavity 
armlength = 3995;   
R_itm = 1970;
R_etm = 2190;
lambda = 1.064e-6;
zETM = 2;                % this is the distance from the ETM to Telescope

lensmakers = @(n, R1, R2, d) ( (n-1)*(1/R1 -1/R2 + (n-1)*d/(n*R1*R2)) )^-1;

[z0,z1,z2] = cavHG(armlength, R_itm, R_etm); % The waist from by the arm


%% Telescope design
goo = beamPath;
goo.seedWaist(sqrt(z0*lambda/pi),-z2-zETM);

% Put down fixed components.  OAPs and mirror are treated as lenses
f_ETM = lensmakers(1.45, -R_etm, inf, 0.2);
goo.addComponent(component.lens(f_ETM, -zETM, 'ETM'));
goo.addComponent(component.lens(2.0,0,'Tel1'));
goo.addComponent(component.lens(0.125, 2.1276,'Tel2'));
goo.addComponent(component.lens(0.344, 3, 'Gouy1'));
goo.addComponent(component.lens(-0.114, 3.243   , 'Gouy2'));

% To put down the QPDs, we find the output waist location and go +/- Zr
zout = 3.5;
qout = goo.qPropagate(zout);
waist_z = zout - qout.waistZ;

goo.addComponent(component.lens(inf, waist_z-qout.rayleighRange, 'QPD1'));
goo.addComponent(component.lens(inf, waist_z+qout.rayleighRange, 'QPD2'));



%% Plot figure
figure(101)
zdomain = -2.1:.01:5;
clf;
subplot(2,1,1)
hold on;
orighandle=goo.plotBeamWidth(zdomain);
goo.plotComponents(zdomain)
goo.plotBeams(zdomain)
hold off;
axis([zdomain(1) zdomain(end) -7e-2 7e-2]);
ylabel('Beam Width (m)')
xlabel('Propagation axis (m)')

subplot(2,1,2)
hold on;
orighandle=goo.plotBeamWidth(zdomain);
goo.plotComponents(zdomain)
goo.plotBeams(zdomain)
hold off;
axis([zdomain(1) zdomain(end) -5e-3 5e-3]);

ylabel('Beam Width (m)')
xlabel('Propagation axis (m)')


%% print the component list to the command window
disp(' ')
disp(' Optimized Path Component List:')
display(goo.components)

%% And calculate the QPD sensitivities
disp(' QPD sensitivity');

QPD_from_Arm = QPD_TransferMatrix(goo, -z2-zETM)
Arm_from_QPD = inv(QPD_from_Arm)

ABCD_qpd1 = goo.getTransferMatrix(-0.5, goo.component('QPD1').z);
ABCD_qpd2 = goo.getTransferMatrix(-0.5, goo.component('QPD2').z);

QPD_from_Table = QPD_TransferMatrix(goo, -0.5)
Arm_from_Table = Arm_from_QPD * QPD_from_Table

%% Calculate the QPD tolerances

figure(201)
subplot(2,1,1)
dZ = -4e-3:3e-5:4e-3;
[gTelescope, zTelOut, fTelOut] = gouyLengthScan(goo, dZ, 'Tel1');

set(gca, 'Fontsize', 16);
plot(dZ*1000, gTelescope, 'LineWidth', 3);
xlabel('BRT length shift [mm]');
ylabel('Gouy phase [deg]');
axis([-4 4 0 100])

subplot(2,1,2)
gGouyTel = gouyLengthScan(goo, dZ, 'Gouy1');

set(gca, 'Fontsize', 16);
plot(dZ*1000, gGouyTel, 'LineWidth', 3);
xlabel('Gouy Telescope length shift [mm]');
ylabel('Gouy phase [deg]');
axis([-4 4 0 100])

figure(2001)
subplot(3,1,1)
plot(dZ*1000, zTelOut)
xlim([-1 1])
ylabel('Waist position [m]');
title('BRT output tuning');

subplot(3,1,2)
plot(dZ*1000, fTelOut*1000)
xlim([-1 1])
ylabel('Waist size [mm]');

subplot(3,1,3)
myZr  = pi*fTelOut.^2/1.064e-6;
plot(dZ*1000, zTelOut./myZr);
xlim([-1 1])
ylabel('Normalized waist pos. [ m / Z_R]');
xlabel('BRT Length shift [mm]');


%% Calculate the aperture required
figure(301)

dz = -0.1:1e-2:5;
zstart = -z2 -zETM;
ETM_spot_shift = 0.01;
cavity_angle_shift = ETM_spot_shift/z2;
num_waist_clear = 6;

for ii = 1:length(dz)
    ABCD = goo.getTransferMatrix(zstart, dz(ii));
    this_waist = goo.qPropagate(dz(ii)).beamWidth;
    
    WaistAperture(ii) = num_waist_clear*this_waist;
        
    PosAperture(ii) = 2*abs(ETM_spot_shift*ABCD(1,1)) + WaistAperture(ii);
    AngAperture(ii) = 2*abs(cavity_angle_shift*ABCD(1,2)) + WaistAperture(ii);
end

set(gca, 'FontSize', 16);
plot(dz,  WaistAperture,dz, AngAperture, dz, PosAperture, ...
    dz, max(PosAperture, AngAperture)/sqrt(2));
legend('6x waist size', 'From position', 'From angle', 'For Green beam');
xlabel('Position [m]');
ylabel('Required aperture [m]');

axis([-1 5 0 0.05]);

%% Calculate the allowable variations.
Ngrid = 21;
Xgrid = 1e-5;
dX = linspace(-Xgrid, Xgrid, Ngrid);
nX = length(dX);
[z_BRT, z_Gouy] = meshgrid(dX);
zTelStart = goo.component('Tel1').z;
zGouyStart =  goo.component('Gouy1').z;
z_BRT = z_BRT +   zTelStart;
z_Gouy = z_Gouy + zGouyStart;

q_cavity = goo.qPropagate(-z2-zETM);
meritscale = q_cavity.divergenceAngle / q_cavity.waistSize ;
clear figmerit;
for ii = 1:nX
    for jj = 1:nX
        goo.component('Tel1').z = z_BRT(ii,jj);
        goo.component('Gouy1').z = z_Gouy(ii,jj);
        
        ABCD = QPD_TransferMatrix(goo, -z2-zETM);
        Mact = Arm_from_QPD*ABCD;
        figmerit(ii,jj) = Mact(1,2)/Mact(1,1) * meritscale;
    end
end

goo.component('Tel1').z = zTelStart;
goo.component('Gouy1').z = zGouyStart;
%%
figure(401)
set(gca, 'FontSize', 16);
imagesc(dX*1e6, dX*1e6, figmerit);
xlabel('\delta BRT length [\mum]');
ylabel('\delta Gouy telescope length [\mum]');
title('Ratio Off-Diagonal to Displacement');
colorbar;
set(gca, 'YDir', 'normal')
p =  polyfit(dX, figmerit(16,:), 1);
fprintf(1, 'Off diagonal term = %.1f%% per BRT um\n', 100*p(1)/1e6);
p =  polyfit(dX, figmerit(:,16).', 1);
fprintf(1, 'Off diagonal term = %.1e%% per Gouy telescope um\n', 100*p(1)/1e6);

val = [-0.01 0.01];
for ii = 1:2
    xline(1) = interp1(figmerit(1,:), dX, val(ii));
    xline(2) = interp1(figmerit(end,:), dX, val(ii));
    yline(1) = dX(1);
    yline(2) = dX(end);
    line(xline*1e6, yline*1e6, 'Color', [0 0 0], 'LineWidth', 2)
end



