%% Calculate the telescope focusing
%
% The telescope focusing consists of an input beam, beam expander, 
% a beam splitter,  the telescope, a retroreflector, the telescope, 
% and a beam expander. 
function TelescopeFocusing

    figure(501);
    clf;
    telfoc = beampath;
    telfoc.seedWaist(1e-3, -1);  % seed the test with a 1 mm beam 1 m in front 
                                 % of the secondary
    f2 = 0.125 - 72e-6;
    zTel = 2.1276;

    zBulkhead = (29 + 4) * 25.4e-3;   % Invar is 29", plus a few for slop
    zExpand = -0.25;  % position of the input/output beam expanders

    telfoc = makeTelescope(f2, zTel, zBulkhead);
    
    % Start with a reasonable beam
    telfoc.seedWaist(1e-3, zExpand-0.05);  
    
    % Input 5x beam expander
    telfoc.addComponent(component.lens(-18e-3, -zExpand, 'Exp1'));
    telfoc.addComponent(component.lens(90e-3, -zExpand+72e-3, 'Exp2'));
    zLast = telfoc.component('Tel2_2').z;
    
    % output 2x beam compressor
    telfoc.addComponent(component.lens(inf, zBulkhead + zLast, 'Bulk'));
    telfoc.addComponent(component.lens(0.10, zBulkhead + zLast -zExpand, 'Exp3'));
    telfoc.addComponent(component.lens(-0.050, 0.050 + zBulkhead + zLast -zExpand, 'Exp4'));
    
    % Here's the nominal parameters
    f0 = findOutputFocus(telfoc);
    fprintf(1,'Nominal output waist = %.0f um\n', ...
        telfoc.qPropagate(10).waistSize*1e6);
    nomRay = telfoc.qPropagate(10).rayleighRange;
    fprintf(1,'Nominal Rayleigh range = %.0f mm\n', ...
        nomRay*1e3);
    telfoc.addComponent(component.lens(inf, f0, 'Target'));
       
    
    
    z = -1:0.01:9;
    telfoc.plotBeamWidth(z);
    hold on;
    telfoc.plotComponents(z);
    hold off;
    focal_length =  10-telfoc.qPropagate(10).waistZ - telfoc.component('Tel2_2').z;
    xlabel('Beam Z [m]');
    ylabel('Beam width [m]');
    
    % The sensitivity to the telescope length
    figure(601)
    clf;
    dFocus = -6e-4:3e-5:6e-4;
    clear focii dtel;
    zStart = [telfoc.component('Tel2_1').z telfoc.component('Tel2_2').z];
    for ii = 1:length(dFocus)
        detuneTelescope(telfoc, dFocus(ii), zStart );
        focii(ii) = findOutputFocus(telfoc);
        dtel(ii) = telfoc.component('Tel2_1').z;
    end
    % Put the telescope back where it started
    detuneTelescope(telfoc, 0, zStart);
    
    subplot(2,1,1);
    plot(dFocus*1e6, (focii-f0)*1000);
    line([-500 -500], [-200 200], 'LineStyle', '--', 'Color', [0 0 0]);
    line([200 200], [-200 200], 'LineStyle', '--', 'Color', [0 0 0]);
    xlabel('Telescope detuning [\mu m]');
    ylabel('Test focus shift [mm]');
    axis([-600 600 -200 200]);

    subplot(2,1,2);
    plot(dFocus*1e6, (focii-f0)/nomRay);
    line([-500 -500], [-200 200], 'LineStyle', '--', 'Color', [0 0 0]);
    line([200 200], [-200 200], 'LineStyle', '--', 'Color', [0 0 0]);
    xlabel('Telescope detuning [\mu m]');
    ylabel('Shift [normalized by Z_R]');
    axis([-600 600 -10 10]);
    
    
    % The sensitivity to the input beam
    dZ_input = -1:0.1:0.5;
    for ii = 1:length(dZ_input)
        telfoc.seedWaist(1e-3, zExpand-0.05 + dZ_input(ii));  
        input_focii(ii) = findOutputFocus(telfoc);
    end
    
    dF_input = 5e-4:1e-4:1.5e-3;
    for ii = 1:length(dF_input)
        telfoc.seedWaist(dF_input(ii), zExpand-0.05);  
        input_focusSize(ii) = findOutputFocus(telfoc);
    end
    telfoc.seedWaist(1e-3, zExpand-0.05);
    
    dZ_inBRT = [-1e-3:1e-4:1e-3];
    zStart = telfoc.component('Exp1').z;
    for ii = 1:length(dZ_inBRT)
        telfoc.component('Exp1').z = zStart + dZ_inBRT(ii);
        input_BRT(ii) = findOutputFocus(telfoc);
    end
    telfoc.component('Exp1').z = zStart;
    
    dZ_outBRT = [-1e-3:1e-4:1e-3];
    zStart = telfoc.component('Exp4').z;
    for ii = 1:length(dZ_inBRT)
        telfoc.component('Exp4').z = zStart + dZ_outBRT(ii);
        output_BRT(ii) = findOutputFocus(telfoc);
    end
    telfoc.component('Exp4').z = zStart;
    
    dZ_posBRT = [-1e-2:1e-3:1e-2];
    zStart = [telfoc.component('Exp3').z telfoc.component('Exp4').z];
    for ii = 1:length(dZ_posBRT)
        telfoc.component('Exp3').z = zStart(1) + dZ_outBRT(ii);
        telfoc.component('Exp4').z = zStart(2) + dZ_outBRT(ii);
        pos_BRT(ii) = findOutputFocus(telfoc);
    end
    telfoc.component('Exp3').z = zStart(1);
    telfoc.component('Exp4').z = zStart(2);
        
    figure(701)
    subplot(3,2,1)
    plot(dZ_input*1000, (input_focii-f0)*1000);
    xlabel('Input focus position [mm]');
    ylabel('Focus shift [mm]');
    ylim([-50 50]);
    
    subplot(3,2,2)
    plot(dF_input*1000, (input_focusSize-f0)*1000);
    xlabel('Input beam waist [mm]');
    ylim([-50 50]);
    
    
    subplot(3,2,3);
    plot(dZ_inBRT*1000, (input_BRT-f0)*1000);
    xlabel('Input BRT length [mm]');
    ylabel('Focus shift [mm]');
    axis([-0.5 0.5 -50 50]);
    
    
    subplot(3,2,4);
    plot(dZ_outBRT*1000, (output_BRT-f0)*1000);
    xlabel('Output BRT length [mm]');
    axis([-0.5 0.5 -50 50]);
 
    subplot(3,2,5)
    plot(dZ_posBRT*1000, (pos_BRT-f0)*1000);
    xlabel('Output BRT position [mm]');
    ylabel('Focus shift [mm]');
    axis([-0.5 0.5 -50 50]);
    

end

function telOut = makeTelescope(f2, zTel, zBulkhead)
% a telescope positioned with the input bulkhead as 0

    f1 = 2.0;
    telOut = beampath;    
    telOut.addComponent(component.lens(f2, zBulkhead, 'Tel2_1'));
    telOut.addComponent(component.lens(f1, zTel+zBulkhead, 'Tel1_1'));
    telOut.addComponent(component.lens(inf, zTel + 2*zBulkhead, 'Retro'));
    telOut.addComponent(component.lens(f1, zTel+3*zBulkhead, 'Tel1_2'));
    telOut.addComponent(component.lens(f2, 2*zTel+3*zBulkhead, 'Tel2_2'));
end

function  detuneTelescope(in, dZ, zStart)
    in.component('Tel2_1').z = zStart(1) - dZ;
    in.component('Tel2_2').z = zStart(2) + dZ;
end

function focus = findOutputFocus(in)
    zMax = 10;
    q = in.qPropagate(zMax);
    focus = 10 - q.waistZ;
end
