%% TransmonABCD.m     SJW 08/06/2009
%
% This calculates the ABCD matrix for the Transmon table laid out in
% the  D0900446-v7 zemax file.  Obviously,  I have necessarily simplified
% the schematic with the intent to check limiting apertures,  etc. layout
% is:
%
%  ETM -> Tel1 -> Tel2 -> Lens1 -> Lens2 -> QPD1 -> QPD2
%
% We calculate the limiting apertures at each place,  and the largest beam
% displacements and angles everywhere.  We eventually calculate the
% sensitivities to displacement for gouy phase readout.
%
%  ITM ROC = 1970 m
%  ETM ROC = 2190 m
%  ETM thickness = 20.4 cm
%  t1->t2 = 1936.1 mm
%  t1->l1 = 937.0 mm 
%  l1back ->l2front = 220.0 mm
%  l2back->Q1 = 315 mm
%  Q2->Q2 = 740 mm

% Starting beam
[z0,z1,z2] = cavHG(3995, 1970, 2190);      % Waist is defined by the arm cavity
q_input = waist2q(sqrt(z0*1.064e-6/pi), 0);    
 
%% Function definitions
dist = @(x) [1 x(1); 0 1];                           % distance
mirror = @(R) [1 0; -2/R(1) 1];                      % radius
dielectric = @(n1,n2,R) [1 0; (n1-n2)/(R*n2) n1/n2]; % n1, n2, R
                                                     % for a dielectric

%r1, r2, thickness.  R1 > 0 for convex,  R1 < 0 for concave.  R2 vice
%versa.  Assumed fused silica
lens = @(r1, r2, th) dielectric(1.45, 1, r2) * dist(th) * ...
       dielectric(1, 1.45, r1);
% Matrix transformation
qNew = @(q,ABCD) (q*ABCD(1,1)+ABCD(1,2)) / (q*ABCD(2,1)+ABCD(2,2));

%% %%%%%%%%  ABCD matrices
toTelescope = lens(-2070, inf, 0.204)*dist(2115);
telescope = mirror(-0.133/cos(AOI))*dist(1.9361)*mirror(4.000)*toTelescope;
firstlens = lens(0.1545, inf, 4e-3)*dist(0.937)*telescope;
secondlens = lens(inf, 0.0644,2e-3)*dist(0.220)*firstlens;
q1  = dist(0.315)*secondlens;
q2 = dist(0.740)*q1;

disp('===========-------+++++++++++++++++++++++------=============');
disp('Arm cavity waist to QPD basis: [ Q1; Q2 ] = M * [ x; theta ]');
waist_to_QPD = [q1(1,:); q2(1,:)]

table2m2 = dist(1.9361)*mirror(4.000);
table2firstlens = dist(0.937)*mirror(-0.133)*table2m2;
table2secondlens =  dist(0.220)*lens(0.1545, inf, 4e-3)*table2firstlens;
table2q1 = dist(0.315)*lens(inf, 0.0644, 2e-3)*table2secondlens;
table2q2 = dist(0.74)*table2q1;

figure(1)
subplot(6,1,1)
zz = 2115:0.001:2120;
plot(zz,q2waist(qofzed(q_input, zz)))
title('input')
subplot(6,1,2)
zz = 0:0.001:5;
plot(zz,q2waist(qofzed(qNew(q_input, telescope), zz)))
title('Telescope')
subplot(6,1,3)
zz = 0:0.001:1;
plot(zz,q2waist(qofzed(qNew(q_input, firstlens), zz)))
title('first lens')
subplot(6,1,4)
plot(zz,q2waist(qofzed(qNew(q_input, secondlens), zz)))
title('second lens')
subplot(6,1,5)
plot(zz,q2waist(qofzed(qNew(q_input, q1), zz)))
title('qpd1')
subplot(6,1,6)
plot(zz,q2waist(qofzed(qNew(q_input, q2), zz)))
 title('qpd2')

 